from pytest import fixture
from datetime import date

from notify.calendar.calendar import Calendar


@fixture
def calendar() -> Calendar:
    return Calendar([
        "exclude_all",
        "include_weekdays",
        {
            "name": "exclude_holidays",
            "options": {
                "country": "HU"
            }
        },
        {
            "name": "exclude_dow",
            "options": {
                "day": 2
            }
        }
    ])


def test_weekday_detection(calendar: Calendar) -> None:
    assert calendar.match_day(date(2022, 7, 22)) is True
    assert calendar.match_day(date(2022, 7, 23)) is False
    assert calendar.match_day(date(2022, 7, 25)) is True


def test_holiday_detection(calendar: Calendar) -> None:
    # 12.26 is a monday but it is Christmas so it should not be a workday
    assert calendar.match_day(date(2022, 12, 26)) is False
    assert calendar.match_day(date(2022, 12, 27)) is True


def test_midweek(calendar: Calendar) -> None:
    assert calendar.match_day(date(2022, 9, 13)) is True
    assert calendar.match_day(date(2022, 9, 14)) is False
    assert calendar.match_day(date(2022, 9, 15)) is True
