from typing import Any
from .rule import CalendarRule


def load_rules(rule_config_list: list[Any]) -> list[CalendarRule]:
    rules: list[CalendarRule] = []

    for rule_config in rule_config_list:
        if isinstance(rule_config, str):
            rule_config = {
                "name": rule_config
            }
        rules.append(load_rule(rule_config))

    return rules


def load_rule(rule_config: dict[str, Any]) -> CalendarRule:
    options = rule_config["options"] if "options" in rule_config else {}

    if rule_config["name"] == "exclude_all":
        from .exclude_all import ExcludeAll
        return ExcludeAll(**options)
    elif rule_config["name"] == "include_weekdays":
        from .include_weekdays import IncludeWeekdays
        return IncludeWeekdays(**options)
    elif rule_config["name"] == "exclude_holidays":
        from .exclude_holidays import ExcludeHolidays
        return ExcludeHolidays(**options)
    elif rule_config["name"] == "include_days":
        from .include_days import IncludeDays
        return IncludeDays(**options)
    elif rule_config["name"] == "exclude_dow":
        from .exclude_dow import ExcludeDayOfWeek
        return ExcludeDayOfWeek(**options)
    elif rule_config["name"] == "exclude_range":
        from .exclude_range import ExcludeRange
        return ExcludeRange(**options)
    else:
        raise ValueError("Unknown rule: " + rule_config["name"])
