from typing import Optional
from .channel import NotificationChannel
from ..messages.notification import Notification

from redmail.email.sender import EmailSender


class EmailChannel(NotificationChannel):
    def __init__(
        self, smtp_username: str, smtp_password: str, smtp_server: str,
        smtp_port: int, from_email: str, to_emails: list[str],
        local_hostname: Optional[str] = None
    ) -> None:
        self._from_email = from_email
        self._to_emails = to_emails

        self._client = EmailSender(
            host=smtp_server,
            port=smtp_port,
            username=smtp_username,
            password=smtp_password,
            local_hostname=local_hostname
        )

    def send_notification(self, message: Notification) -> None:
        try:
            self._client.connect()  # type: ignore
            for to_email in self._to_emails:
                print("Sending emails to", to_email)
                try:
                    self._client.send(
                        subject=message.title,
                        sender=self._from_email,
                        receivers=to_email,
                        text=message.message,
                    )
                except Exception as e:
                    print(f"Failed to send email: {e}")
            self._client.close()  # type: ignore
        except Exception as e:
            print(f"Failed to send email: {e}")
