import requests
import urllib.parse

from .channel import NotificationChannel
from ..messages.notification import Notification


class MatrixMessageChannel(NotificationChannel):
    def __init__(self, bot_server: str, instance_id: str, secret: str, room_ids: list[str]) -> None:
        super().__init__()
        self._bot_server = bot_server
        self._instance_id = instance_id
        self._secret = secret
        self.room_ids = room_ids

    def send_notification(self, message: Notification) -> None:
        for room_id in self.room_ids:
            print("Sending message to room", room_id)

            e_instance_id = urllib.parse.quote(self._instance_id)
            e_room_id = urllib.parse.quote(room_id)
            url = f"https://{self._bot_server}/_matrix/maubot/plugin/{e_instance_id}/webhook/r0?room={e_room_id}"

            try:
                response = requests.post(
                    url,
                    json={
                        "secret": self._secret,
                        "message": message.message
                    }
                )
                response.raise_for_status()
            except Exception as e:
                print(f"Failed to send matrix message: {e}")
