import nexmo

from .channel import NotificationChannel
from ..messages.notification import Notification


class VonageSmsChannel(NotificationChannel):
    def __init__(self, key: str, secret: str, sender: str, to_numbers: list[str]) -> None:
        super().__init__()
        self._client = nexmo.Client(key=key, secret=secret)
        self._numbers = to_numbers
        self._sender = sender

    def send_notification(self, message: Notification) -> None:
        for number in self._numbers:
            print("Sending SMS to", number)
            try:
                self._client.send_message({
                    'from': self._sender,
                    'to': number,
                    'text': message.short_message
                    if message.short_message else message.message
                })
            except Exception as e:
                print(f"Failed to send SMS: {e}")
