from typing import Any, Generator, Union
import yaml

from deepmerge import always_merger

from .channels.channel import NotificationChannel
from .channels.vonage_sms import VonageSmsChannel
from .channels.email import EmailChannel
from .channels.matrix import MatrixMessageChannel

CHANNEL_TYPES = {
    "sms": VonageSmsChannel,
    "email": EmailChannel,
    "matrix": MatrixMessageChannel
}


def load_config(files: Union[str, list[str]]) -> Any:
    config: Any = {}

    if isinstance(files, str):
        files = [files]

    for file in files:
        print("# Loading config from", file)
        with open(file, "r") as infile:
            config = always_merger.merge(config, yaml.safe_load(infile))

    return config


def build_channels(config: Any) -> Generator[NotificationChannel, None, None]:
    for channel in config["channels"]:
        type = channel["type"]
        if type not in CHANNEL_TYPES:
            raise ValueError(f"Unknown channel type: {type}")
        yield CHANNEL_TYPES[type](**channel["config"])
