from typing import Any
from pytest import fixture
from datetime import date

from notify.timeline.iterator import TimelineIterator
from notify.timeline.basic import BasicTimelineIterator, FixedDay
from notify.calendar.calendar import Calendar
from notify.messages.generator import build_notification


@fixture
def basic_timeline_iterator() -> BasicTimelineIterator:
    return BasicTimelineIterator(
        workday_calendar=Calendar([
            "exclude_all",
            "include_weekdays",
            {
                "name": "exclude_holidays",
                "options": {
                    "country": "HU"
                }
            },
        ]),
        fixed_day=FixedDay(
            day=date(2022, 9, 21),
            moderator="John",
            days_left=2
        ),
        moderator_days=3,
        moderators=["John", "Jane", "Jack"]
    )


@fixture
def template_config() -> Any:
    return {
        "message": {
            "title": "Daily standup",
            "normal_template": "daily.j2",
            "short_template": "daily-short.j2"
        }
    }


def test_weekend(basic_timeline_iterator: TimelineIterator, template_config: Any) -> None:
    msg = build_notification(basic_timeline_iterator, template_config, date(2022, 9, 25))
    assert msg is None


def test_message_generation(basic_timeline_iterator: TimelineIterator, template_config: Any) -> None:
    msg = build_notification(basic_timeline_iterator, template_config, date(2022, 9, 21))
    assert msg is not None
    assert msg.title == "Daily standup"
    assert msg.message == """Dear Team Tirith!

Today John will hold the daily meeting!

The upcoming schedule for the next 10 workdays is as follows:
    2022-09-22 (Thursday): John
    2022-09-23 (Friday): John
    2022-09-26 (Monday): Jane
    2022-09-27 (Tuesday): Jane
    2022-09-28 (Wednesday): Jane
    2022-09-29 (Thursday): Jack
    2022-09-30 (Friday): Jack
    2022-10-03 (Monday): Jack
    2022-10-04 (Tuesday): John
    2022-10-05 (Wednesday): John
Please be aware that this schedule might change due to special circumstances.

Have a nice day!"""

    assert msg.short_message == """Dear Team Tirith!

Today John will hold the daily meeting!

Have a nice day!"""
