from typing import Any, Optional
import jinja2
from datetime import date

from ..timeline.iterator import TimelineIterator, CalendarDay
from .notification import Notification


def render(tempalte_file: str, data: Any) -> str:
    templateLoader = jinja2.FileSystemLoader(searchpath="./templates")
    templateEnv = jinja2.Environment(loader=templateLoader)
    template = templateEnv.get_template(tempalte_file)
    return template.render(data)


def day_to_data(day: CalendarDay) -> dict[str, str]:
    return {
        **day.__dict__,
        "dow": day.day.strftime('%A')
    }


def build_notification(iter: TimelineIterator, config: Any, starting_day: date) -> Optional[Notification]:
    iter.set_day(starting_day)

    days: list[CalendarDay] = []
    for day in iter:
        days.append(day)
        if len(days) >= 11:
            break

    if len(days) == 0 or days[0].day != starting_day:
        return None

    template_data = {
        "today": day_to_data(days[0]) if len(days) >= 1 else None,
        "days": [day_to_data(day) for day in days[1:]]
    }

    title = config["message"]["title"]
    normal_msg_template_file = config["message"]["normal_template"]
    short_msg_template_file = config["message"]["short_template"]

    return Notification(
        title=title,
        message=render(normal_msg_template_file, template_data),
        short_message=render(short_msg_template_file, template_data)
        if short_msg_template_file else None
    )
