from pytest import fixture
from datetime import date

from notify.timeline.basic import BasicTimelineIterator, FixedDay
from notify.timeline.iterator import CalendarDay
from notify.calendar.calendar import Calendar


@fixture
def basic_timeline() -> BasicTimelineIterator:
    return BasicTimelineIterator(
        workday_calendar=Calendar([
            "exclude_all",
            "include_weekdays",
            {
                "name": "exclude_holidays",
                "options": {
                    "country": "HU"
                }
            },
        ]),
        fixed_day=FixedDay(
            day=date(2022, 9, 21),
            moderator="a",
            days_left=2
        ),
        moderator_days=3,
        moderators=["a", "b", "c"]
    )


def test_fixed_day(basic_timeline: BasicTimelineIterator) -> None:
    basic_timeline.set_day(date(2022, 9, 21))
    assert basic_timeline.__next__() == CalendarDay(day=date(2022, 9, 21), moderator="a")


def test_iteration(basic_timeline: BasicTimelineIterator) -> None:
    expected_items = [
        CalendarDay(day=date(2022, 9, 21), moderator="a"),
        CalendarDay(day=date(2022, 9, 22), moderator="a"),
        CalendarDay(day=date(2022, 9, 23), moderator="a"),
        CalendarDay(day=date(2022, 9, 26), moderator="b"),
        CalendarDay(day=date(2022, 9, 27), moderator="b"),
        CalendarDay(day=date(2022, 9, 28), moderator="b"),
        CalendarDay(day=date(2022, 9, 29), moderator="c"),
    ]

    basic_timeline.set_day(date(2022, 9, 21))

    for day in basic_timeline:
        expected = expected_items.pop(0)
        assert day == expected

        if len(expected_items) == 0:
            break


def test_far_in_future(basic_timeline: BasicTimelineIterator) -> None:
    expected_items = [
        CalendarDay(day=date(2046, 9, 21), moderator="c"),
        CalendarDay(day=date(2046, 9, 24), moderator="c"),
        CalendarDay(day=date(2046, 9, 25), moderator="c"),
        CalendarDay(day=date(2046, 9, 26), moderator="a"),
    ]

    basic_timeline.set_day(date(2046, 9, 21))

    for day in basic_timeline:
        expected = expected_items.pop(0)
        assert day == expected

        if len(expected_items) == 0:
            break
