from dataclasses import dataclass
from datetime import date, timedelta

from .iterator import TimelineIterator, CalendarDay
from ..calendar.calendar import Calendar


@dataclass
class FixedDay():
    day: date
    moderator: str
    days_left: int


class BasicTimelineIterator(TimelineIterator):
    def __init__(self,
                 workday_calendar: Calendar, fixed_day: FixedDay,
                 moderators: list[str], moderator_days: int) -> None:
        self._workday_calendar = workday_calendar
        self._fixed_day = fixed_day
        self._moderators = moderators
        self._moderator_days = moderator_days

        self._offset_days = self._get_initial_offset()

    def _get_initial_offset(self) -> int:
        return \
            self._moderators.index(self._fixed_day.moderator) * self._moderator_days + \
            self._moderator_days - self._fixed_day.days_left - 1

    def _count_days(self, from_date: date, target_date: date) -> int:
        from_date = from_date
        iterator = from_date
        date_counter = 0

        while iterator < target_date:
            iterator += timedelta(days=1)
            if self._workday_calendar.match_day(iterator):
                date_counter += 1

        return date_counter

    def set_day(self, day: date) -> None:
        self._check_day = day - timedelta(days=1)
        self._quit_day = day + timedelta(days=365)

    def __next__(self) -> CalendarDay:
        while True:
            self._check_day += timedelta(days=1)

            if self._check_day > self._quit_day:
                raise StopIteration

            if self._workday_calendar.match_day(self._check_day):
                count = self._count_days(self._fixed_day.day, self._check_day)
                moderator_id = (count + self._offset_days) // self._moderator_days

                return CalendarDay(
                    day=self._check_day,
                    moderator=self._moderators[moderator_id % len(self._moderators)]
                )
